package org.gcube.portlets.user.geoportaldataviewer.client.ui.cms.project;

import org.gcube.application.geoportalcommon.shared.GeoNaItemRef;
import org.gcube.application.geoportalcommon.shared.geoportal.view.ProjectView;
import org.gcube.application.geoportalcommon.shared.geoportal.view.SectionView;
import org.gcube.portlets.user.geoportaldataviewer.client.GeoportalDataViewerConstants;
import org.gcube.portlets.user.geoportaldataviewer.client.GeoportalDataViewerConstants.MAP_PROJECTION;
import org.gcube.portlets.user.geoportaldataviewer.client.GeoportalDataViewerServiceAsync;
import org.gcube.portlets.user.geoportaldataviewer.client.ui.dialogs.DialogShareableLink;
import org.gcube.portlets.user.geoportaldataviewer.client.ui.map.ExtentMapUtil;
import org.gcube.portlets.user.geoportaldataviewer.client.ui.map.ExtentMapUtil.Location;
import org.gcube.portlets.user.geoportaldataviewer.client.ui.map.ExtentMapUtil.PLACE;
import org.gcube.portlets.user.geoportaldataviewer.client.ui.map.MapView;
import org.gcube.portlets.user.geoportaldataviewer.client.ui.util.CustomFlexTable;

import com.github.gwtbootstrap.client.ui.Button;
import com.github.gwtbootstrap.client.ui.PageHeader;
import com.github.gwtbootstrap.client.ui.constants.ButtonType;
import com.github.gwtbootstrap.client.ui.constants.IconType;
import com.google.gwt.core.client.GWT;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.uibinder.client.UiBinder;
import com.google.gwt.uibinder.client.UiField;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.HTMLPanel;
import com.google.gwt.user.client.ui.Widget;

import ol.Coordinate;

public class ProjectViewer extends Composite {

	private static ProjectViewerUiBinder uiBinder = GWT.create(ProjectViewerUiBinder.class);

	interface ProjectViewerUiBinder extends UiBinder<Widget, ProjectViewer> {
	}

	@UiField
	PageHeader title;

	@UiField
	HTMLPanel pageViewDetails;

	@UiField
	Button shareButton;

	@UiField
	Button expandButton;

	private ProjectView projectView;

	private CustomFlexTable customTable = new CustomFlexTable();

	private GeoNaItemRef geonaItemRef;

	private String myLogin;

	private boolean viewImageButtonVisible = true;

	private boolean openImageButtonVisible = true;

	private ProjectViewer() {
		initWidget(uiBinder.createAndBindUi(this));
		pageViewDetails.getElement().setId("page-view-details");
	}

	public ProjectViewer(GeoNaItemRef item, ProjectView projectView) {
		this(item, projectView, true, true);
	}

	public ProjectViewer(GeoNaItemRef item, ProjectView projectView, boolean viewImageButtonVisible,
			boolean openImageButtonVisible) {
		this();
		GWT.log("Rendering " + projectView.getTheProjectDV().getId());
		this.projectView = projectView;
		this.geonaItemRef = item;
		this.viewImageButtonVisible = viewImageButtonVisible;
		this.openImageButtonVisible = openImageButtonVisible;

		title.setText("Project: "+projectView.getTheProjectDV().getProfileID());
		
		shareButton.setType(ButtonType.LINK);
		shareButton.setIcon(IconType.SHARE);
		shareButton.setTitle("Get a link to share with...");

		shareButton.addClickHandler(new ClickHandler() {

			@Override
			public void onClick(ClickEvent event) {
				DialogShareableLink dg = new DialogShareableLink(geonaItemRef, null);

			}
		});

		expandButton.setType(ButtonType.LINK);
		expandButton.setIcon(IconType.EXPAND);
		expandButton.setTitle("Show this view in new Window");

		expandButton.addClickHandler(new ClickHandler() {

			@Override
			public void onClick(ClickEvent event) {
				Window.alert("Expand must be revisited");
				/*
				ProjectViewer cv = new ProjectViewer(geonaItemRef, concessioneDV, false, openImageButtonVisible);
				cv.setExpandViewButtonVisible(false);
				int width = Window.getClientWidth() * 75 / 100;
				int height = Window.getClientHeight() * 70 / 100;
				ModalWindow mw = new ModalWindow(concessioneDV.getNome(), width, height);
				mw.add(cv);
				mw.setCaller(ProjectViewer.this);
				// mw.setWidth(900);
				mw.show();
				*/

			}
		});



		GeoportalDataViewerServiceAsync.Util.getInstance().getMyLogin(new AsyncCallback<String>() {

			@Override
			public void onSuccess(String result) {
				myLogin = result;
				addCentroidMap();
			}

			@Override
			public void onFailure(Throwable caught) {
			}
		});
		
		
		for (SectionView sectionView : projectView.getListSections()) {
			
			SectionViewer sectionViewer = new SectionViewer(sectionView);
			pageViewDetails.add(sectionViewer);
		}
	}

	private void addCentroidMap() {
		Location italyLocation = ExtentMapUtil.getLocation(PLACE.ITALY);
		Coordinate transformedCenterCoordinate = italyLocation.getCoordinate(MAP_PROJECTION.EPSG_3857);
		MapView mapView = new MapView(transformedCenterCoordinate,
				GeoportalDataViewerConstants.LIGHT_MAP_ITALY_FIT_ZOOM_ON, "70%", "300px");

		Window.alert("addCentroidMap must be revisited");
		
//		if (concessioneDV != null && concessioneDV.getCentroidLat() != null
//				&& concessioneDV.getCentroidLong() != null) {
//			Coordinate coord = new Coordinate(concessioneDV.getCentroidLong(), concessioneDV.getCentroidLat());
//			Coordinate transfCoord = MapUtils.transformCoordiante(coord, MAP_PROJECTION.EPSG_4326.getName(),
//					MAP_PROJECTION.EPSG_3857.getName());
//			// Coordinate invertedCoordinate = MapUtils.reverseCoordinate(coord);
//			boolean authenticatedUser = myLogin != null ? true : false;
//			mapView.addMarker(transfCoord, authenticatedUser);
//			mapViewPanel.add(mapView);
//		} else if (concessioneDV != null) {
//			GeoportalDataViewerConstants
//					.printJs("I cannot add centroid as maker one or both coordinates are null. Lat: "
//							+ concessioneDV.getCentroidLong() + ", Long:" + concessioneDV.getCentroidLat());
//		}
	}

	public ProjectView getProjectView() {
		return projectView;
	}

	protected void setExpandViewButtonVisible(boolean bool) {
		expandButton.setVisible(bool);
	}

}
